/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/* 
 * $Id: conditional.h,v 1.9 2002/06/15 15:16:13 mulix Exp $
 */

#ifndef CONDITIONAL_H_
#define CONDITIONAL_H_

#include "conditional_intf.h"
#include "param_types.h"

/* forward reference */
class parser;

/* a class which describes an (old style, deprecated) conditional clause */ 
class conditional: public conditional_intf
{
 public:
    /* constructor */ 
    conditional();

    /* copy constructor */ 
    conditional(const conditional& rhs);

    /* virtual destructor */ 
    virtual ~conditional();

    /* validate this rule for this syscall. */ 
    virtual bool validate(const string& syscall_name);

    /* allow subclasses to override these, just in case */
    virtual void set_operator(const string& str);
    virtual void set_operator(oper_type_t op);
    virtual oper_type_t get_operator() const;

    virtual void set_line_num(int line_num) {_line_num = line_num;}
    virtual int get_line_num() {return _line_num;}
    virtual void set_data_line_num(int line_num) {_data_line_num = line_num;}
    virtual int get_data_line_num() {return _data_line_num;}

    virtual void set_value(const string& str);
    virtual const value_type_t& get_value() const;

    ostream& print(ostream& os);

    bool finalize(const string& sys_name);

 protected:
    /* returns a filter node made of 'lhs op rhs', where caller */
    /* supplies the rhs                                         */
    sct_filter_node_t* sct_get_filter_node(sct_filter_node_t* rhs) const;

 protected:
    oper_type_t _op;
    value_type_t _val;
    string _tmp_val;
    int _line_num;
    int _data_line_num;

 private:
    /* disallow assignment */ 
    conditional& operator=(const conditional& rhs); 
};

#endif /* CONDITIONAL_H_ */
