/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/*
 *$Id: logger.cpp,v 1.7 2002/06/15 15:16:13 mulix Exp $
 */

#include <string>
#include <iostream>

#include "logger.h"

#include <syslog.h>
#include <stdarg.h>
#include <stdio.h>

bool logger::_use_syslog = false;
logger::verbosity_level_t logger::_verbosity_level = LOG_LEVEL_DEBUG;


void logger::do_log(const char* msg)
{
    if (logger::_use_syslog){
	syslog(LOG_ERR, msg);
    }else{
	cerr<<msg<<flush;
    }
}

void logger::log(verbosity_level_t vl, const string& msg)
{
    if (vl >= _verbosity_level){
	do_log(msg.c_str());
    }
}

void logger::log(verbosity_level_t vl, const char* fmt, ...)
{
    if (vl >= _verbosity_level){
	char buff[1024];
	va_list ap;
	va_start(ap, fmt);
	vsnprintf(buff, sizeof(buff), fmt, ap);
	do_log(buff);
	va_end(ap);
    }
}

void logger::open_syslog(const char* progname)
{
    openlog(progname, LOG_PID, LOG_NOTICE | LOG_USER);
}

void logger::close_syslog()
{
    closelog();
}
