/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/*
 * $Id: param_conditional.h,v 1.17 2002/06/15 15:16:13 mulix Exp $
 */

/* This is the a condition which is based upon a parmeter to the syscall
 */

#ifndef _PARAM_CONDITIONAL_H_
#define _PARAM_CONDITIONAL_H_

#include <string>
#include <iostream>

#include "conditional.h"
#include "param_types.h"
#include "syscalls.h"

using namespace std;

class param_conditional: public conditional
{
 public:
    param_conditional();
    ~param_conditional();
    param_conditional(const param_conditional& rhs);

    bool validate(const string& syscall_name);
    
    void set_location(const string& str);
    unsigned short int get_location() const;

    void set_location_field(const string& str);
    const string& get_location_field() const;

    void set_location_field_index(int field_index);
    int get_location_field_index() const;

    sct_filter_node_t* sct_get_filter_node(void) const;

    ostream& print(ostream& os);

    bool finalize(const string& sys_name);

 protected:
    sct_data_type_t query_required_data_type(const string& sys_name);
    
 private:
    bool is_valid_for_finalization(const string& sys_name);

    unsigned short int _loc;
    string _loc_field;
    int _loc_field_index;
};

#endif //_PARAM_CONDITIONAL_H_
