/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/*
 *$Id: param_types.h,v 1.19 2002/06/15 15:16:13 mulix Exp $
 */

#ifndef _PARAM_TYPES_H_
#define _PARAM_TYPES_H_

using namespace std;

#include <iostream>
#include <string>

#include "sct_rules/rule_engine_public.h"

typedef enum {
    OT_EQUAL = 0,
    OT_LESS,
    OT_GREATER,
    OT_LESS_EQUAL,
    OT_GREATER_EQUAL,
    OT_NOT_EQUAL,
    OT_PLUS,
    OT_MINUS,
    OT_BIT_SHIFT_LEFT,
    OT_BIT_SHIFT_RIGHT,
    OT_BIT_AND,
    OT_BIT_OR,
    OT_BIT_XOR,
    OT_BIT_NOT,
    OT_LOGIC_AND,
    OT_LOGIC_OR,
    OT_LOGIC_NOT,
    OT_PATTERN_MATCH,
    OT_LAST_OPERATOR
} oper_type_t;

typedef enum {
    PT_PID = 0,
    PT_PPID,
    PT_UID,
    PT_EUID,
    PT_SUID,
    PT_GID, 
    PT_EGID, 
    PT_SGID, 
    PT_COMM,
    PT_PCOMM,
    PT_LAST_PROCESS_FIELD
} process_field_t;

union value_data_t{
    int b;                 /* bool   */
    char c;                /* char   */
    int i;                 /* int    */
    unsigned int ui;       /* uint   */
    long l;                /* long   */
    unsigned long ul;      /* ulong  */
    short s;               /* short  */
    unsigned short us;     /* ushort */
    float f;               /* float  */
    char* str;             /* string */
};

struct value_type_t {
    sct_data_type_t type;
    value_data_t val;
};

typedef enum {
    ACT_LOG = 0,
    ACT_FAIL,
    ACT_KILL,
    ACT_SUSPEND,
    ACT_LAST_ACT
} parser_sct_action_t;

typedef enum {
    COND_ANY = 0,
    COND_ALL,
    COND_LAST_COND
} condition_matching_t;

ostream& operator<<(ostream& os, const value_type_t& vt);
ostream& operator<<(ostream& os, const oper_type_t& op);
ostream& operator<<(ostream& os, const process_field_t& proc);

value_type_t* clone_value_type(const value_type_t* val);
bool copy_value_type(value_type_t& val_to, const value_type_t& val_from);
void free_value_type(value_type_t* val);

sct_filter_node_t* 
value_type_t_to_filter_node(const value_type_t& val);

sct_filter_node_t*
operator_to_filter_node(oper_type_t op, sct_filter_node_t* t1, sct_filter_node_t* t2);

sct_filter_node_t* 
process_field_t_to_var(process_field_t proc);

#endif /* _PARAM_TYPES_H_ */

