/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/* 
 * $Id: parse_exception.cpp,v 1.5 2002/06/15 15:16:13 mulix Exp $
 */

#include "parse_exception.h"

basic_exception::basic_exception(const string& conf_file, 
				 unsigned int conf_line, 
				 const string& err):
    _conf_fname(conf_file), _conf_line(conf_line), _err(err)
{
}

basic_exception::~basic_exception()
{
}

void 
basic_exception::print(ostream& out) const
{
	std::string stdin("stdin");

        /* hack for '-', which stands for stdin */
	const std::string & name = 
		      ((_conf_fname.length() == 1 && _conf_fname[0] == '-')
		      ? stdin : _conf_fname); 

	out << name << ":" << _conf_line << " " << _err << endl;
}

debug_exception::debug_exception(const string& fname = "", 
				 const string& func = "", 
				 unsigned int line = 0):
    _conf_name(fname), _func(func), _line(line)
{
}

debug_exception::~debug_exception()
{
}

void 
debug_exception::print(ostream& out) const
{
    out<<"["<<_conf_name<<":"<<_line<<":"<<_func<<"()] ";
}


basic_parse_exception::basic_parse_exception(const string& conf_file, 
					     unsigned int conf_line, 
					     const string& err):
    basic_exception(conf_file, conf_line, err),
    _has_token(false), _t()
{
}

basic_parse_exception::basic_parse_exception(const string& conf_file, 
					     unsigned int conf_line, 
					     const string& err, 
					     const token& tok):
    basic_exception(conf_file, conf_line, err),
    _has_token(true), _t(tok)
{
}


basic_parse_exception::~basic_parse_exception()
{
}

void 
basic_parse_exception::print(ostream& out) const
{
    basic_exception::print(out);
    if (_has_token) 
	out<<_t;
}

debug_parse_exception::debug_parse_exception(const string& conf_file, 
					     unsigned int conf_line, 
					     const string& err, 
					     const string& fname = "",
					     const string& func = "", 
					     unsigned int line = 0):
    basic_parse_exception(conf_file, conf_line, err),
    _d(fname, func, line)
{
}

debug_parse_exception::debug_parse_exception(const string& conf_file, 
					     unsigned int conf_line, 
					     const string& err, 
					     const token& t, 
					     const string& fname = "",
					     const string& func = "", 
					     unsigned int line = 0):
    basic_parse_exception(conf_file, conf_line, err, t),
    _d(fname, func, line)
{
}

debug_parse_exception::~debug_parse_exception()
{
}

void 
debug_parse_exception::print(ostream& out) const
{
    _d.print(out);
    basic_parse_exception::print(out);
}


basic_validate_exception::basic_validate_exception(const string& conf_file, 
						   unsigned int conf_line, 
						   const string& err):
    basic_exception(conf_file, conf_line, err)
{
}

basic_validate_exception::~basic_validate_exception()
{
}

void 
basic_validate_exception::print(ostream& out) const
{
    basic_exception::print(out);
}


debug_validate_exception::debug_validate_exception(const string& conf_file, 
						   unsigned int conf_line, 
						   const string& err, 
						   const string& fname = "",
						   const string& func = "", 
						   unsigned int line = 0):
    basic_validate_exception(conf_file, conf_line, err),
    _d(fname, func, line)
{
}

debug_validate_exception::~debug_validate_exception()
{
}

void 
debug_validate_exception::print(ostream& out) const
{
    _d.print(out);
    basic_validate_exception::print(out);
}

