/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/*
 *$Id: process_conditional.cpp,v 1.21 2002/06/15 15:16:13 mulix Exp $
 */

#include <iostream>

#include "parser.h"
#include "parse_exception.h"
#include "process_conditional.h"
#include "validity_checks.h"
#include "token.h"
#include "tokens_to_types.h"
#include "logger.h"
#include "param_types.h"

process_conditional::process_conditional():
    conditional(), _proc_field(PT_LAST_PROCESS_FIELD)
{
}

process_conditional::~process_conditional()
{
}

process_conditional::process_conditional(const process_conditional& rhs)
    :conditional(rhs)
{
    _proc_field = rhs._proc_field;
}

ostream& 
process_conditional::print(ostream& os)
{
    os<<"processs conditional: "<<endl;
    os<<"\t\tprocess_field: "<<_proc_field<<" ";
    return conditional::print(os)<<endl;
}

void 
process_conditional::set_process_field(const string& str)
{
    _proc_field = tokens::get_process_field(str);
    if (_proc_field == PT_LAST_PROCESS_FIELD) {
 	string err = "'" + str + "' is an invalid process field";
	throw parse_exception("", get_line_num(), err);
    }
}

bool 
process_conditional::validate(const string& syscall_name)
{
    if (!conditional::validate(syscall_name)){
	/* error logging is done in conditional::is_valid() */
	return false; 
    }

    /* check the process field */
    if (_proc_field == PT_LAST_PROCESS_FIELD){
	strstream buf;
	buf << "invalid process field for syscall '" << syscall_name << "'\0";

	throw validate_exception("", get_line_num(), buf.str());
    }

    return true;
}

process_field_t
process_conditional::get_process_field(void) const
{
    return _proc_field;
}

sct_filter_node_t* 
process_conditional::sct_get_filter_node(void) const
{
    sct_filter_node_t* rhs = process_field_t_to_var(_proc_field);
    if (!rhs)
	return NULL; 

    return conditional::sct_get_filter_node(rhs);
}

bool
process_conditional::finalize(const string& sys_name)
{
    return conditional::finalize(sys_name);
}

sct_data_type_t 
process_conditional::query_required_data_type(const string& sys_name)
{
    sct_data_type_t res = DT_LAST_TYPE;

    switch (_proc_field){
    case (PT_PID): 
    case (PT_PPID): res = tokens::get_data_type("pid_t"); break;
    case (PT_UID):
    case (PT_EUID):
    case (PT_SUID):
    case (PT_GID):
    case (PT_EGID):
    case (PT_SGID): res = tokens::get_data_type("uid_t"); break;
    case (PT_COMM):
    case (PT_PCOMM): res = tokens::get_data_type("string"); break;
    default:
	break;
    }

    return res;
}
