/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/*
 *$Id: token.h,v 1.19 2002/06/15 15:16:13 mulix Exp $
 */

#ifndef _TOKEN_H_
#define _TOKEN_H_

#include <string>

using namespace std;

typedef enum {
    PR_RULE = 0,
    PR_LOG_FORMAT,
    PR_LOG_FORMAT_BEFORE,
    PR_LOG_FORMAT_AFTER,
    PR_LOG_FORMAT_DEFAULT, 
    PR_ACTION,
    PR_ACT_TYPE, 
    PR_ACT_SETPARAMATTR, 
    PR_ACT_SETPARAMATTR_VAR_INDEX, 
    PR_ACT_SETPARAMATTR_ATTR_NAME, 
    PR_ACT_SETPARAMATTR_ATTR_VAL, 
    PR_ACT_ERROR_CODE, 
    PR_ACT_LOG_PRIO, 
    PR_ACT_PID, 
    PR_SYSCALL_NAME,
    PR_RULE_NAME,
    PR_RULE_WHEN,
    PR_PARAM_LOCATION,
    PR_PARAM_LOCATION_FIELD,
    PR_CONDITIONAL, 
    PR_PROCESS_FIELD,
    PR_OPERATOR,
    PR_VALUE,
    PR_PARAM_CONDITION,
    PR_PROCESS_CONDITION,
    PR_CONDITIONS,
    PR_FILTER_EXPRESSION, 
    PR_LAST_TOKEN_TYPE
} parser_token_t;

ostream& operator<<(ostream& os, const parser_token_t& t);

class token
{
 public:
    token(): _type(PR_LAST_TOKEN_TYPE), _value() {}
    token(const token& rhs): _type(rhs._type), _value(rhs._value), 
	    _type_line_num(rhs._type_line_num), 
	    _value_line_num(rhs._value_line_num) {}

    parser_token_t get_type() const {return _type;}
    void set_type(parser_token_t type){_type = type;}
    const string& get_value() const {return _value;}
    void set_value(const string& value){_value = value;}
    void set_type_line_num(int line_num) {_type_line_num = line_num;}
    int get_type_line_num() const {return _type_line_num;}
    void set_value_line_num(int line_num) {_value_line_num = line_num;}
    int get_value_line_num() const {return _value_line_num;}

    friend ostream& operator<<(ostream& os, const token& t);
 private:
    parser_token_t _type;
    string _value;
    int _type_line_num;
    int _value_line_num;

};

#endif /* _TOKEN_H_ */
