/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/* 
 * $Id: tokens_to_types.cpp,v 1.24 2002/06/15 15:16:13 mulix Exp $
 */

#include "tokens_to_types.h"
#include "param_types.h"

bool tokens::initialized = false;
tokens::operators_type tokens::operators;
tokens::data_types_type tokens::data_types;
tokens::data_types_str_type tokens::data_types_str;
tokens::process_fields_type tokens::process_fields;
tokens::parser_tokens_type tokens::parser_tokens;
tokens::actions_type tokens::actions;
tokens::var_attr_name_type tokens::var_attr_names;
tokens::conditions_matching_type tokens::conditions;

tokens::tokens()
{
}

tokens::~tokens()
{
}

void tokens::init()
{
    if (tokens::initialized) 
	return;

    /* initialize the operators map */
    tokens::operators["=="] = OT_EQUAL;
    tokens::operators["<"]  = OT_LESS;
    tokens::operators[">"]  = OT_GREATER;
    tokens::operators["<="] = OT_LESS_EQUAL;
    tokens::operators[">="] = OT_GREATER_EQUAL;
    tokens::operators["!="] = OT_NOT_EQUAL;
    tokens::operators["+"]  = OT_PLUS;
    tokens::operators["-"]  = OT_MINUS;
    tokens::operators["<<"] = OT_BIT_SHIFT_LEFT;
    tokens::operators[">>"] = OT_BIT_SHIFT_RIGHT;
    tokens::operators["&"]  = OT_BIT_AND;
    tokens::operators["|"]  = OT_BIT_OR;
    tokens::operators["^"]  = OT_BIT_XOR;
    tokens::operators["~"]  = OT_BIT_NOT;
    tokens::operators["&&"] = OT_LOGIC_AND;
    tokens::operators["||"] = OT_LOGIC_OR;
    tokens::operators["!"]  = OT_LOGIC_NOT;
    tokens::operators["~="] = OT_PATTERN_MATCH;
    tokens::operators["=~"] = OT_PATTERN_MATCH;

    /* initialize the data_types map */
    tokens::data_types["b"]               = DT_BOOL;
    tokens::data_types["c"]               = DT_CHAR;
    tokens::data_types["char"]            = DT_CHAR;
    tokens::data_types["i"]               = DT_INT;
    tokens::data_types["int"]             = DT_INT;
    tokens::data_types["ui"]              = DT_UINT;
    tokens::data_types["uint"] 		  = DT_UINT;
    tokens::data_types["unsigned_int"]    = DT_UINT;
    tokens::data_types["s"]               = DT_SHORT;
    tokens::data_types["short"]           = DT_SHORT;
    tokens::data_types["us"]              = DT_USHORT;
    tokens::data_types["ushort"]  	  = DT_USHORT;
    tokens::data_types["unsigned_short"]  = DT_USHORT;
    tokens::data_types["l"]               = DT_LONG;
    tokens::data_types["long"]            = DT_LONG;
    tokens::data_types["ul"]              = DT_ULONG;
    tokens::data_types["ulong"]   	  = DT_ULONG;
    tokens::data_types["unsigned_long"]   = DT_ULONG;
    tokens::data_types["f"]               = DT_FLOAT;
    tokens::data_types["float"]           = DT_FLOAT;
    tokens::data_types["str"]             = DT_STRING;
    tokens::data_types["string"]          = DT_STRING;
    tokens::data_types["const_string"]    = DT_STRING;
    tokens::data_types["buffer"]          = DT_BUFFER; 
    tokens::data_types["const_buffer"]    = DT_BUFFER; 
    tokens::data_types["mode_t"]          = DT_USHORT;
    tokens::data_types["pid_t"]           = DT_LONG;
    tokens::data_types["size_t"]          = DT_ULONG; 
    tokens::data_types["buffer_size"]     = DT_ULONG; 
    tokens::data_types["ssize_t"]         = DT_LONG; 
    tokens::data_types["buffer_size"]     = DT_LONG; 
    tokens::data_types["off_t"]	          = DT_LONG;
    tokens::data_types["uid_t"]           = DT_USHORT;
    tokens::data_types["net_ushort"]      = DT_USHORT;
    tokens::data_types["net_ip"]          = DT_ULONG;
    tokens::data_types["dev_t"]           = DT_USHORT; 
    tokens::data_types["ino_t"]           = DT_ULONG; 
    tokens::data_types["link_t"]          = DT_USHORT;
    tokens::data_types["blksize_t"]       = DT_ULONG; 
    tokens::data_types["blkcnt_t"]        = DT_ULONG;
    tokens::data_types["clock_t"]         = DT_LONG;

    /* data types -> string require special handling, 
       since it's not a one to one mapping */
    tokens::data_types_str[DT_BOOL]   = "bool";
    tokens::data_types_str[DT_CHAR]   = "char";
    tokens::data_types_str[DT_INT]    = "int";
    tokens::data_types_str[DT_UINT]   = "unsigned int";
    tokens::data_types_str[DT_SHORT]  = "short";
    tokens::data_types_str[DT_USHORT] = "unsigned short";
    tokens::data_types_str[DT_LONG]   = "long";
    tokens::data_types_str[DT_ULONG]  = "unsigned long";
    tokens::data_types_str[DT_FLOAT]  = "float";
    tokens::data_types_str[DT_STRING] = "string";
    tokens::data_types_str[DT_BUFFER] = "buffer"; 
        
    /* initialize the process fields map */
    tokens::process_fields["PPID"] =  PT_PPID;
    tokens::process_fields["PID"]  =  PT_PID;
    tokens::process_fields["UID"]  =  PT_UID;
    tokens::process_fields["EUID"] =  PT_EUID;
    tokens::process_fields["SUID"] =  PT_SUID;
    tokens::process_fields["GID"]  =  PT_GID;
    tokens::process_fields["EGID"] =  PT_EGID;
    tokens::process_fields["SGID"] =  PT_SGID;
    tokens::process_fields["COMM"] =  PT_COMM; 
    tokens::process_fields["PCOMM"] =  PT_PCOMM;
    
    tokens::parser_tokens["rule"]                 = PR_RULE;
    tokens::parser_tokens["log_format"]           = PR_LOG_FORMAT;
    tokens::parser_tokens["before"]               = PR_LOG_FORMAT_BEFORE;
    tokens::parser_tokens["after"]                = PR_LOG_FORMAT_AFTER;
    tokens::parser_tokens["default"]              = PR_LOG_FORMAT_DEFAULT;
    tokens::parser_tokens["syscall_name"]         = PR_SYSCALL_NAME;
    tokens::parser_tokens["rule_name"]            = PR_RULE_NAME;
    tokens::parser_tokens["when"]            	  = PR_RULE_WHEN;
    tokens::parser_tokens["param_location"]       = PR_PARAM_LOCATION;
    tokens::parser_tokens["param_location_field"] = PR_PARAM_LOCATION_FIELD;
    tokens::parser_tokens["process_field"]        = PR_PROCESS_FIELD;
    tokens::parser_tokens["action"]               = PR_ACTION;
    tokens::parser_tokens["type"]                 = PR_ACT_TYPE; 
    tokens::parser_tokens["error_code"]           = PR_ACT_ERROR_CODE; 
    tokens::parser_tokens["priority"]             = PR_ACT_LOG_PRIO;
    tokens::parser_tokens["pid"]                  = PR_ACT_PID; 
    tokens::parser_tokens["operator"]             = PR_OPERATOR;
    tokens::parser_tokens["value"]                = PR_VALUE;
    tokens::parser_tokens["param_condition"]      = PR_PARAM_CONDITION;
    tokens::parser_tokens["process_condition"]    = PR_PROCESS_CONDITION;
    tokens::parser_tokens["conditions"]           = PR_CONDITIONS;
    tokens::parser_tokens["filter_expression"]    = PR_FILTER_EXPRESSION;

    tokens::actions["LOG"]     = ACT_LOG;
    tokens::actions["FAIL"]    = ACT_FAIL;
    tokens::actions["SUSPEND"] = ACT_SUSPEND;
    tokens::actions["KILL"]    = ACT_KILL;

    tokens::parser_tokens["set_param_attr"]       = PR_ACT_SETPARAMATTR; 
    tokens::parser_tokens["attr_param"]       = PR_ACT_SETPARAMATTR_VAR_INDEX; 
    tokens::parser_tokens["attr_name"]        = PR_ACT_SETPARAMATTR_ATTR_NAME; 
    tokens::parser_tokens["attr_val"]         = PR_ACT_SETPARAMATTR_ATTR_VAL; 

    tokens::var_attr_names["var_dyn_type"]    = VAN_DYN_TYPE;

    tokens::conditions["ANY"]  = COND_ANY;
    tokens::conditions["ALL"]  = COND_ALL;
    
    initialized = true;
}

oper_type_t
tokens::get_operator(const string& str)
{
    tokens::operators_type::const_iterator it = 
	tokens::operators.find(str);
    if (it == tokens::operators.end())
	return OT_LAST_OPERATOR;
    
    return (*it).second;
}
    
sct_data_type_t
tokens::get_data_type(const string& str)
{
    tokens::data_types_type::const_iterator it = 
	tokens::data_types.find(str);
    if (it == tokens::data_types.end())
	return DT_LAST_TYPE;
    
    return (*it).second;
}    

process_field_t
tokens::get_process_field(const string& str)
{
    tokens::process_fields_type::const_iterator it = 
	tokens::process_fields.find(str);
    if (it == tokens::process_fields.end())
	return PT_LAST_PROCESS_FIELD;
    
    return (*it).second;
}

parser_sct_action_t 
tokens::get_action(const string& str)
{
    tokens::actions_type::const_iterator it = 
	tokens::actions.find(str);
    if (it == tokens::actions.end())
	return ACT_LAST_ACT;
    
    return (*it).second;
}

sct_var_attr_name_t 
tokens::get_var_attr(const string& str)
{
    tokens::var_attr_name_type::const_iterator it = 
	tokens::var_attr_names.find(str);
    if (it == tokens::var_attr_names.end())
	return VAN_LAST_TYPE;
    
    return (*it).second;
}

condition_matching_t 
tokens::get_condition_matching(const string& str)
{
    tokens::conditions_matching_type::const_iterator it = 
	tokens::conditions.find(str);
    if (it == tokens::conditions.end()) {
	return COND_LAST_COND;
    }
    
    return (*it).second;
}

parser_token_t 
tokens::get_parser_token(const string& str)
{
    tokens::parser_tokens_type::const_iterator it = 
	tokens::parser_tokens.find(str);
    if (it == tokens::parser_tokens.end())
	return PR_LAST_TOKEN_TYPE;
    
    return (*it).second;
}

 
string 
tokens::get_operator_str(oper_type_t op)
{
    tokens::operators_type::const_iterator it = 
	tokens::operators.begin();
    for ( ; it != tokens::operators.end(); ++it) {
	if ((*it).second == op)
	    return (*it).first;
    }
    return "(unknown)";
}

string 
tokens::get_data_type_str(sct_data_type_t dt)
{
    /* first of all look for it in the data_types_str map */
    tokens::data_types_str_type::const_iterator it = 
	tokens::data_types_str.find(dt);

    if (it != tokens::data_types_str.end())
	return (*it).second;
    
    /* look in the regular map? */
    tokens::data_types_type::const_iterator it2 = 
	tokens::data_types.begin();
    for ( ; it2 != tokens::data_types.end(); ++it2) {
	if ((*it2).second == dt)
	    return (*it2).first;
    }

    return "(unknown)";
}
 
string
tokens::get_process_field_str(process_field_t pr)
{
    tokens::process_fields_type::const_iterator it = 
	tokens::process_fields.begin();
    for ( ; it != tokens::process_fields.end(); ++it) {
	if ((*it).second == pr)
	    return (*it).first;
    }
    return "(unknown)";
}

string
tokens::get_parser_token_str(parser_token_t pr)
{
    tokens::parser_tokens_type::const_iterator it = 
	tokens::parser_tokens.begin();
    for ( ; it != tokens::parser_tokens.end(); ++it) {
	if ((*it).second == pr)
	    return (*it).first;
    }
    return "(unknown)";
}


string 
tokens::get_action_str(parser_sct_action_t act)
{
    tokens::actions_type::const_iterator it = 
	tokens::actions.begin();
    for ( ; it != tokens::actions.end(); ++it) {
	if ((*it).second == act)
	    return (*it).first;
    }
    return "(unknown)";
}

string 
tokens::get_action_var_attr_name_str(sct_var_attr_name_t name)
{
    tokens::var_attr_name_type::const_iterator it = 
	tokens::var_attr_names.begin();
    for ( ; it != tokens::var_attr_names.end(); ++it) {
	if ((*it).second == name)
	    return (*it).first;
    }
    return "(unknown)";
}

string 
tokens::get_condition_matching_str(condition_matching_t match)
{
    tokens::conditions_matching_type::const_iterator it = 
	tokens::conditions.begin();
    for ( ; it != tokens::conditions.end(); ++it) {
	if ((*it).second == match) 
	    return (*it).first;
    }
    return "(unknown)";
}
