/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU General Public License                              */

/*
 * $Id: tokens_to_types.h,v 1.6 2002/06/15 15:16:13 mulix Exp $
 */

#ifndef TOKENS_TO_TYPES_
#define TOKENS_TO_TYPES_

#include <map>
#include <string>

using namespace std;

#include "param_types.h"
#include "token.h"

class tokens {
 public: /* public methods */
    tokens();
    ~tokens();

    /* initialize */
    static void init();

    /* map from string to the enumerated type */
    static oper_type_t get_operator(const string& str);
    static sct_data_type_t get_data_type(const string& str); 
    static process_field_t get_process_field(const string& str);
    static parser_token_t get_parser_token(const string& str);
    static parser_sct_action_t get_action(const string& str);
    static sct_var_attr_name_t get_var_attr(const string& str);
    static condition_matching_t get_condition_matching(const string& str);

    
    /* map from enumerated type to string representation */
    static string get_operator_str(oper_type_t op); 
    static string get_data_type_str(sct_data_type_t dt);
    static string get_process_field_str(process_field_t pr);
    static string get_parser_token_str(parser_token_t pr);
    static string get_action_str(parser_sct_action_t act);
    static string get_action_var_attr_name_str(sct_var_attr_name_t name);
    static string get_condition_matching_str(condition_matching_t match);

 public: /* public typedefs */
    typedef map <string, oper_type_t> operators_type;
    typedef map <string, sct_data_type_t> data_types_type;
    typedef map <string, process_field_t> process_fields_type;
    typedef map <string, parser_token_t> parser_tokens_type;
    typedef map <string, parser_sct_action_t> actions_type;
    typedef map <string, sct_var_attr_name_t> var_attr_name_type;
    typedef map <string, condition_matching_t> conditions_matching_type;
    typedef map <sct_data_type_t, string> data_types_str_type;

 private:
    static operators_type operators;
    static data_types_type data_types;
    static process_fields_type process_fields;
    static parser_tokens_type parser_tokens;
    static actions_type actions;
    static var_attr_name_type var_attr_names;
    static conditions_matching_type conditions;
    static data_types_str_type data_types_str;

    static bool initialized;
};

#endif /* TOKENS_TO_TYPES_ */
