/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU Lesser General Public License                       */

/*
 * $Id: dev_file_impl.h,v 1.7 2002/06/15 15:16:13 mulix Exp $
 */ 

#ifndef DEV_FILE_IMPL_H_
#define DEV_FILE_IMPL_H_

/* this file implements the device file communication protocol between
   the kernel module and userspace. for protocol details, see 
   doc/sct_device_control_file_design.txt */

#include "sct_ctrl_lib.h"
#include "sct_ctrl_lib_private.h"

/* add rule */
int 
dev_file_add_rule(sct_ctrl_t ctrl,
		  int rule_id,
		  sct_rule_when_t when,
		  int syscall_id, 
		  sct_filter_node_t* filter, sct_action_t* act,
		  int enable_rule);

/* delete a rule */
int
dev_file_del_rule(sct_ctrl_t ctrl,
		  int rule_id,
		  sct_rule_when_t when,
		  int syscall_id);

/* enable a rule */
int 
dev_file_enable_rule(sct_ctrl_t ctrl,
		     int rule_id,
		     sct_rule_when_t when,
		     int syscall_id);

/* disable a rule */
int 
dev_file_disable_rule(sct_ctrl_t ctrl,
		      int rule_id,
		      sct_rule_when_t when,
		      int syscall_id);

/* delete all rules for this syscall */
int 
dev_file_del_all_syscall_rules(sct_ctrl_t ctrl, int syscall_id);

/* delete all rules for ALL syscalls */
int 
dev_file_del_all_rules(sct_ctrl_t ctrl);

/* print the current rules */
int
dev_file_print_all_rules(sct_ctrl_t ctrl);

/* get all rule count. returns -1 on failure or the rule count */ 
int
dev_file_get_all_rule_count(sct_ctrl_t ctrl); 

/* get all of the rules. return -1 on error or the number of lists in the list */ 
int
dev_file_get_all_rules(sct_ctrl_t ctrl, syscall_rules_t* p_rules); 

extern sct_ctrl_ops_t dev_file_ops; 

#endif /* DEV_FILE_IMPL_H_ */
