/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU Lesser General Public License                       */

/*
 * $Id: sct_ctrl_lib.h,v 1.15 2002/06/15 15:16:13 mulix Exp $
 */

#ifndef SCT_CTRL_LIB_H_
#define SCT_CTRL_LIB_H_

#ifdef __cplusplus
extern "C" {
#endif
    
#include "sct_rules/rule_engine_public.h"
    
/* the opaque pointer all sct_ctrl_lib operations use */
typedef struct sct_ctrl_type* sct_ctrl_t; 

typedef enum {
    SCT_CTRL_DEFAULT = 0, 
    SCT_CTRL_SYSCTL, 
    SCT_CTRL_DEVICE_FILE
} sct_ctrl_impl_t;

/* an 'all rules for this system call' data type */ 
typedef struct syscall_rules {
    int syscall_id; /* which syscall this rule list is for */ 
    sct_rule_when_t when; /* is it for before or after? */ 
    sct_rule_list_t* list; /* the list of rules */ 
    struct syscall_rules* next; /* the next list node, NULL at the end */ 
} syscall_rules_t; 

/* initialize the library.                 */
/* pass SCT_COMM_DEFAULT for comm type if  */
/* you dont care about it                  */
/* return an sct_ctrl_t, or NULL if failed */
sct_ctrl_t
sct_ctrl_init(sct_ctrl_impl_t sct_impl);

/* shutdown the library.                   */
/* return 0 on failure, non 0 on success   */
int 
sct_ctrl_cleanup(sct_ctrl_t ctrl);

/* add a rule with the given rule id to the given syscall,    */
/* and the given 'when' ('before' or 'after' syscall invoke)  */
/* where the rule is made of this filter and this action      */
/* and 'enable rule' specifies if the rule should be          */
/* immediately enabled or not                                 */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_add_rule(sct_ctrl_t ctrl, int rule_id, sct_rule_when_t when,
		  int syscall_id, 
		  sct_filter_node_t* filter, sct_action_t* act, 
		  int enable_rule);

/* delete from the given syscall the rule with this rule id   */
/* and the given 'when' ('before' or 'after' syscall invoke)  */
/* return 0 on failure, non 0 on success                      */
int
sct_ctrl_del_rule(sct_ctrl_t ctrl,
		  int rule_id,
		  sct_rule_when_t when,
		  int syscall_id);


/* enable the rule with this id for this syscall              */
/* NOTE: currently NOT IMPLEMENTED                            */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_enable_rule(sct_ctrl_t ctrl,
		     int rule_id,
		     sct_rule_when_t when,
		     int syscall_id);

/* disable the rule with this id for this syscall             */
/* NOTE: currently NOT IMPLEMENTED                            */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_disable_rule(sct_ctrl_t ctrl,
		      int rule_id,
		      sct_rule_when_t when,
		      int syscall_id);

/* delete all of the rules for this system call.              */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_del_all_syscall_rules(sct_ctrl_t ctrl, int syscall_id);

/* delete all rules for ALL syscalls.                         */
/* return 0 on failure, non 0 on success                      */
int 
sct_ctrl_del_all_rules(sct_ctrl_t ctrl);

/* print all of the current rules to the system's log file    */
/* return 0 on failure, non 0 on success                      */
int
sct_ctrl_print_all_rules(sct_ctrl_t ctrl);

/* returns the number of defined rules for all system calls */ 
int
sct_ctrl_get_all_rule_count(sct_ctrl_t ctrl);

/* returns a list of currently defined rules in p_rules (out   */ 
/* parameter). returns the number of rule lists in the list or */
/* -1 on error. (a list can have 0 rules)                      */
int
sct_ctrl_get_all_rules(sct_ctrl_t ctrl, syscall_rules_t* p_rules);

#ifdef __cplusplus
}
#endif

#endif /* SCT_CTRL_LIB_H_ */
