/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU Lesser General Public License                       */

/*
 * $Id: sct_ctrl_lib_private.h,v 1.5 2002/06/15 15:16:13 mulix Exp $
 */

#ifndef SCT_CTRL_LIB_PRIVATE_H_
#define SCT_CTRL_LIB_PRIVATE_H_

/* the enum of the different communication methods */
#include "sct_ctrl_lib.h"

typedef struct sct_ctrl_ops {
    sct_ctrl_t (*init)(sct_ctrl_impl_t sct_impl); 
    int (*cleanup)(sct_ctrl_t ctrl);
    int (*add_rule)(sct_ctrl_t ctrl, int rule_id, sct_rule_when_t when,
		    int syscall_id, 
		    sct_filter_node_t* filter, sct_action_t* act, 
		    int enable_rule);
    int (*del_rule)(sct_ctrl_t ctrl, int rule_id,
		    sct_rule_when_t when, int syscall_id);
    int (*enable_rule)(sct_ctrl_t ctrl, int rule_id,
		       sct_rule_when_t when, int syscall_id);
    int (*disable_rule)(sct_ctrl_t ctrl, int rule_id,
			sct_rule_when_t when, int syscall_id);
    int (*del_all_syscall_rules)(sct_ctrl_t ctrl, int syscall_id);
    int (*del_all_rules)(sct_ctrl_t ctrl);
    int (*print_all_rules)(sct_ctrl_t ctrl);
    int (*get_all_rule_count)(sct_ctrl_t ctrl);
    int (*get_all_rules)(sct_ctrl_t ctrl, syscall_rules_t* p_rules);
} sct_ctrl_ops_t;

struct sct_ctrl_type {
    sct_ctrl_impl_t impl;
    sct_ctrl_ops_t* ops; 
    void* user_data;
};

void*
get_user_data(sct_ctrl_t ctrl); 

/* we get a membuf which contains a list of serialized  */
/* rule lists, and we return a list of syscall_rules_t  */ 
/* structures, two lists per syscall, for before and after rules  */
int
parse_rulesbuf(sct_membuf_t* mbuf, syscall_rules_t* p_rules); 

#endif /* SCT_CTRL_LIB_PRIVATE_H_ */
