/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il	    */
/* License: GNU Lesser General Public License                       */

/* 
 * $Id: sysctl_impl.h,v 1.6 2002/06/15 15:16:13 mulix Exp $
 */

#ifndef SYSCTL_IMPL_H_
#define SYSCTL_IMPL_H_

#include "sct_ctrl_lib.h"
	     

/* add rule */
int 
sysctl_add_rule(sct_ctrl_t ctrl, int rule_id, sct_rule_when_t when,
		int syscall_id, 
		sct_filter_node_t* filter, sct_action_t* act,
		int enable_rule);

/* delete a rule */
int
sysctl_del_rule(sct_ctrl_t ctrl, int rule_id, sct_rule_when_t when, int syscall_id);

/* enable a rule */
int 
sysctl_enable_rule(sct_ctrl_t ctrl,
		   int rule_id,
		   sct_rule_when_t when,
		   int syscall_id);

/* disable a rule */
int 
sysctl_disable_rule(sct_ctrl_t ctrl,
		    int rule_id,
		    sct_rule_when_t when,
		    int syscall_id);

/* delete all rules for this syscall */
int 
sysctl_del_all_syscall_rules(sct_ctrl_t ctrl, int syscall_id);

/* delete all rules for ALL syscalls */
int 
sysctl_del_all_rules(sct_ctrl_t ctrl);

/* print the current rules */
int
sysctl_print_all_rules(sct_ctrl_t ctrl);

/* get all of the current rules */
int
sysctl_get_all_rules(sct_ctrl_t ctrl, syscall_rules_t* p_rules); 

/* get the count of all currently defined rules */ 
int
sysctl_get_all_rule_count(sct_ctrl_t ctrl); 

extern sct_ctrl_ops_t sysctl_ops; 

#endif /* SYSCTL_IMPL_H_ */
