/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/* 
 * $Id: buffer_test.c,v 1.3 2002/06/15 15:16:14 mulix Exp $
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "sct_rules/rule_engine_public.h"
#include "sct_rules/rule_engine_private.h"

#include "tester.h"

int test_void_ptr_buffer(void)
{
	char buf[16], *p; 
	int i = sizeof(buf); 

	srand(time(NULL)); 

	p = buf; 
	while (i--) {
		int randval =  1+(int) (254.0*rand()/(RAND_MAX+1.0));
		if (isprint(randval)) {
			printf("randval is %c\n", randval); 
		} else {
			printf("randval is %d\n", randval); 
		}
		*p++ = randval; 
	}

	printf("%s\n", sct_buffer_to_string(buf, sizeof(buf))); 
	return 1; 
}

int test_buffer_match(void)
{
	sct_filter_node_t* b; 
	sct_filter_node_t* match; 
	sct_filter_node_t* pat;
	sct_filter_node_t* res = sct_make_bool_val(TF_FALSE); 
	sct_exp_error_t rc; 

	b = sct_make_string_val("mulix"); 
	match = sct_make_buffer_val("m", 1); 
	pat = sct_make_pattern_match(b, match); 
	
	if ((rc = sct_eval_filter_node(pat, res)) != ERR_NONE) {
		printf("evaluating a patern match returned %d\n", rc); 
		return 0; 
	}

	printf("match result (should be 1): "); 
	sct_print_filter_node(res); 

	sct_free_filter_node(pat); 
	sct_free_filter_node(res); 
	return 1; 
}

int dummymain(void)
{
	test_buffer_match(); 
	return 0; 
}

