#! /usr/bin/env python

# This file is part of syscalltrack, a GNU/Linux kernel module and
# user space utilities for logging or modifying any system call    
# invocation.                                                      
#                                                                  
# Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      
# License: GNU General Public License                              

#
# $ Id: compile_test.py,v 1.3 2001/10/02 17:40:09 mulix Exp $
#

import sys
import os
import string
import re

class SpawnError(Exception):   # the exception we throw if any error occurs
    def __init__(self, value): # constructor
        self.value = value
    def __str__(self):         # string representation
        return `self.value`

def run_command(command):
    """ run a command in the local directory """
    if (command == None): return
    com = command.split(); 
    rc = os.spawnvp(os.P_WAIT, com[0], com); 
    if (rc != 0):
        raise SpawnError, command + " failed"

def compile(c_compiler, cpp_compiler, kernel_dir):
    """
    run 'make_all' in the current directory, using the given
    compilers and setting KERNEL_DIR to 'kernel_dir'
    """
    os.system('./clean_all > /dev/null')
    command = "./make_all GXX=%s CC=%s KERNEL_DIR=%s" % (cpp_compiler, c_compiler, kernel_dir)
    print '\n\ncompiling with: %s \n\n' % (command)
    sys.stdout.flush()
    run_command(command)
    
if __name__ == "__main__":

    compilers_file = open('tests/compilers.dat', "r")
    kernel_dirs_file = open('tests/kernel_dirs.dat', "r")

    compilers = [string.strip(z) for z in compilers_file.readlines()]
    kernel_dirs = [string.strip(z) for z in kernel_dirs_file.readlines()]

    regexp = re.compile("^#|^ ")

    for c in compilers:
        if regexp.match(c):
            continue
        for k in kernel_dirs:
            if regexp.match(k):
                continue
            rc = compile(c+'gcc', c+'g++', k)
            if (rc != 0):
                print '\n\n\ncompilation failed with ',c,'gcc KERNEL_DIR=',k,'\n\n'
                sys.stdout.flush()
    
