/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 guy keren, choo@actcom.co.il             */
/* License: GNU General Public License                              */

/* 
 * $Id: get_rules_test.c,v 1.5 2002/06/15 15:16:14 mulix Exp $
 */ 

/* test the 'get rules' and 'get rule count' functionality, */ 
/* using both sysctl and device file */ 

#include <assert.h>

#include "sct_ctrl_lib/sct_ctrl_lib.h"
#include "tester.h"

sct_ctrl_t sysctl, dev; 

static void
init_sct_ctrl(void)
{
	/* initialize sct_ctrl_lib for sysctl and device file*/ 
	sct_ctrl_impl_t isysctl = SCT_CTRL_SYSCTL; 
	sct_ctrl_impl_t idev = SCT_CTRL_DEVICE_FILE; 

	if (!(sysctl = sct_ctrl_init(isysctl))) {
		die("failed to init sct_ctrl_lib for sysctl\n");
	}

	if (!(dev = sct_ctrl_init(idev))) {
		die("failed to init sct_ctrl_lib for device file\n");
	}

	if (!sct_ctrl_del_all_rules(sysctl))
		die("failed to delete the module rules\n"); 
		
	if (!sct_ctrl_del_all_rules(dev))
		die("failed to delete the module rules\n"); 
}

static void 
print_rule_list(syscall_rules_t* rules)
{
	assert(rules); 

	for ( ; rules; rules = rules->next) {
		char buf[32]; 
		sct_get_syscall_id_as_str(rules->syscall_id, buf, sizeof(buf)); 
		printf(" syscall: %s, %s rules:\n", buf, 
		       (rules->when == RW_BEFORE? "before" : "after")); 
		sct_print_tracking_rules(rules->list); 
	}
}

static int compare_rule_list(syscall_rules_t* r1, syscall_rules_t* r2)
{
	return 1; 
}

int test_get_rules(void)
{
	int rule_count; 
	int rc1, rc2; 

	syscall_rules_t d, s, *di = &d, *si = &s; 

	init_sct_ctrl(); 

	assert(sct_ctrl_get_all_rule_count(sysctl) == 0); 
	assert(sct_ctrl_get_all_rule_count(dev) == 0); 

	/* upload some rules */ 
	rule_count = all_syscalls_register_rules(); 
	
	rc1 = sct_ctrl_get_all_rule_count(sysctl); 
	rc2 = sct_ctrl_get_all_rule_count(dev); 
	printf("test_get_rule_count: rc1 is %d\n", rc1); 
	assert(rc1 == rule_count); 
	assert(rc1 == rc2); 

	/* now get the rules */ 
	rc1 = sct_ctrl_get_all_rules(sysctl, &s); 
	rc2 = sct_ctrl_get_all_rules(dev, &d); 
	printf("test_get_rule_count: rc1 is %d\n", rc1); 
	assert(rc1 == rc2); 
	
	/* compare the two lists */ 
	/* dont do the loop if there are no rules registered */ 
	while (rc1 && si && di) {
		assert(compare_rule_list(di, si)); 
		print_rule_list(si); 
		print_rule_list(di); 
		si = si->next; 
		di = di->next; 
	}
	if (rc1)
		assert(!si && !di); 

	if (!sct_ctrl_del_all_rules(sysctl))
		die("failed to delete the module rules\n"); 
		
	if (!sct_ctrl_del_all_rules(dev))
		die("failed to delete the module rules\n"); 

	sct_ctrl_cleanup(dev); 
	sct_ctrl_cleanup(sysctl); 

	return 1; 
}
