/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/* 
 * $Id: membuf_test.c,v 1.3 2002/06/15 15:16:14 mulix Exp $
 */

#include <assert.h>
#include <stdio.h>

#include "sct_rules/membuf.h"

int test_membuf(void)
{
	char buf[] = "hello, world!"; 
	char buf2[sizeof(buf)]; 
	char scratch[1000] = {0}; 
	int rc; 

	sct_membuf_t* m1; 
	sct_membuf_t* m2; 

	printf("\ntesting memory buffers\n\n\n"); 

	m1 = sct_membuf_create(100, NULL, NULL); 
	m2 = sct_membuf_create_with_data(buf, sizeof(buf), NULL, NULL); 
	assert(m1 && m2); 

	sct_membuf_get_chunk(m2, buf2, sizeof(buf2)); 
	assert(!memcmp(buf, buf2, sizeof(buf))); 

	sct_membuf_add_chunk(m1, buf, sizeof(buf)-1); /* get rid of the NULL */ 
	sct_membuf_add_chunk(m1, buf, sizeof(buf)); 

	printf("m1 now contains: %s\n", sct_membuf_get_data(m1)); 
	printf("removing a chunk of size 6 from m1...\n"); 
	sct_membuf_get_chunk(m1, buf2, 6);
	buf2[6] = '\0';
	printf("chunk is '%s', m2 is '%s'\n", buf2, sct_membuf_get_data(m1)); 

	printf("trying to get a really big chunk...\n"); 
	rc = sct_membuf_get_chunk(m1, scratch, sizeof(scratch)); 
	printf("get_chunk returned '%d', buffer is now '%s'\n", rc, scratch); 

	printf("trying to get a 0 chunk...\n"); 
	rc = sct_membuf_get_chunk(m2, scratch, 0); 
	printf("get_chunk returned '%d'\n", rc);

	printf("trying to push a big chunk to a small buffer\n");
	rc = sct_membuf_add_chunk(m1, scratch, sizeof(scratch)); 
	printf("add_chunk returned '%d'\n", rc);

	sct_membuf_destroy(m1); 
	sct_membuf_destroy(m2); 

	return 1; 
}
