/* This file is part of syscalltrack, a GNU/Linux kernel module and */
/* user space utilities for logging or modifying any system call    */
/* invocation.                                                      */
/*                                                                  */
/* Copyright (C) 2000-2002 Muli Ben-Yehuda, mulix@actcom.co.il      */
/* License: GNU General Public License                              */

/* 
 * $Id: tester.h,v 1.14 2002/06/15 15:16:14 mulix Exp $
 */ 

#ifndef TESTER_H_
#define TESTER_H_

#include <stdio.h>

void 
_die(const char* func, const char* file,
     unsigned int line, const char* fmt, ...);

void 
_logerr(const char* func, const char* file,
	unsigned int line, const char* fmt, ...);

#define die(format, args...) \
	_die(__FUNCTION__,__FILE__,__LINE__, format , ## args)
#define logerr(format, args...) \
	_logerr(__FUNCTION__,__FILE__,__LINE__, format , ## args)

#define TEST(foo) do { fprintf(stderr, "starting test: '%s'\n", #foo); \
                       fflush(stderr); \
		       if (!(foo)) die("test '%s' failed\n!", #foo); \
                       else { \
		           fprintf(stderr, "test '%s' succeeded\n", #foo); \
		           fflush(stderr); \
		       } \
                     } while (0)

/* useful functions we wish to share accross object files */ 
/* sct_rules_module_tests.c: */ 
int all_syscalls_register_rules(void); 

/* module tests */

int test_simple_syscall_usage(void);

int test_rule_register_deregister(void);

int test_all_syscalls_usage(void);

int test_settimeofday_usage(void);

/* sct_rules tests */

int test_filter_eval_bit(void);

int test_eval_complex_filter(void);

int test_filter_serialize_deserialize(void);

int test_rules_serialize_deserialize(void);

int test_tracking_rule_add_delete(void);

int test_tracking_rule_add_delete_empty_list(void);

int test_indexed_field_variables(void);

int test_kernel_variables(void);

int test_short_circuiting(void);

int test_filter_node_vectors(void);

int test_membuf(void); 

int test_tracking_rule_list(void); 

/* "get rules" tests */ 
int test_get_rules(void); 

/* "void* buffer" tests */ 
int test_void_ptr_buffer(void); 

int test_buffer_match(void); 

#endif /* TESTER_H_ */
